@@ -67,13 +67,13 @@ class EnumCallAnalyzer:
         items, values, ok = self.parse_enum_call_args(call, fullname.split('.')[-1])
         if not ok:
             # Error. Construct dummy return value.
-            info = self.build_enum_call_typeinfo(var_name, [], fullname)
+            info = self.build_enum_call_typeinfo(var_name, [], fullname, node.line)
         else:
             name = cast(Union[StrExpr, UnicodeExpr], call.args[0]).value
             if name != var_name or is_func_scope:
                 # Give it a unique name derived from the line number.
                 name += '@' + str(call.line)
-            info = self.build_enum_call_typeinfo(name, items, fullname)
+            info = self.build_enum_call_typeinfo(name, items, fullname, call.line)
             # Store generated TypeInfo under both names, see semanal_namedtuple for more details.
             if name != var_name or is_func_scope:
                 self.api.add_symbol_skip_local(name, info)
@@ -82,10 +82,11 @@ class EnumCallAnalyzer:
         info.line = node.line
         return info
 
-    def build_enum_call_typeinfo(self, name: str, items: List[str], fullname: str) -> TypeInfo:
+    def build_enum_call_typeinfo(self, name: str, items: List[str], fullname: str,
+                                 line: int) -> TypeInfo:
         base = self.api.named_type_or_none(fullname)
         assert base is not None
-        info = self.api.basic_new_typeinfo(name, base)
+        info = self.api.basic_new_typeinfo(name, base, line)
         info.metaclass_type = info.calculate_metaclass_type()
         info.is_enum = True
         for item in items:
@@ -113,7 +114,7 @@ class EnumCallAnalyzer:
         valid_name = [None, 'value', 'names', 'module', 'qualname', 'type', 'start']
         for arg_name in call.arg_names:
             if arg_name not in valid_name:
-                self.fail_enum_call_arg("Unexpected keyword argument '{}'".format(arg_name), call)
+                self.fail_enum_call_arg('Unexpected keyword argument "{}"'.format(arg_name), call)
         value, names = None, None
         for arg_name, arg in zip(call.arg_names, args):
             if arg_name == 'value':
@@ -128,7 +129,7 @@ class EnumCallAnalyzer:
             return self.fail_enum_call_arg(
                 "%s() expects a string literal as the first argument" % class_name, call)
         items = []
-        values = []  # type: List[Optional[Expression]]
+        values: List[Optional[Expression]] = []
         if isinstance(names, (StrExpr, UnicodeExpr)):
             fields = names.value
             for field in fields.replace(',', ' ').split():
