@@ -51,14 +51,14 @@ from mypy.nodes import (
     MypyFile, SymbolTable, Block, AssignmentStmt, NameExpr, MemberExpr, RefExpr, TypeInfo,
     FuncDef, ClassDef, NamedTupleExpr, SymbolNode, Var, Statement, SuperExpr, NewTypeExpr,
     OverloadedFuncDef, LambdaExpr, TypedDictExpr, EnumCallExpr, FuncBase, TypeAliasExpr, CallExpr,
-    CastExpr,
+    CastExpr, TypeAlias,
     MDEF
 )
 from mypy.traverser import TraverserVisitor
 from mypy.types import (
     Type, SyntheticTypeVisitor, Instance, AnyType, NoneType, CallableType, ErasedType, DeletedType,
     TupleType, TypeType, TypeVarType, TypedDictType, UnboundType, UninhabitedType, UnionType,
-    Overloaded, TypeVarDef, TypeList, CallableArgument, EllipsisType, StarType, LiteralType,
+    Overloaded, TypeVarType, TypeList, CallableArgument, EllipsisType, StarType, LiteralType,
     RawExpressionType, PartialType, PlaceholderType, TypeAliasType
 )
 from mypy.util import get_prefix, replace_object_state
@@ -103,7 +103,7 @@ def replacement_map_from_symbol_table(
     the given module prefix. Don't recurse into other modules accessible through the symbol
     table.
     """
-    replacements = {}  # type: Dict[SymbolNode, SymbolNode]
+    replacements: Dict[SymbolNode, SymbolNode] = {}
     for name, node in old.items():
         if (name in new and (node.kind == MDEF
                              or node.node and get_prefix(node.node.fullname) == prefix)):
@@ -188,7 +188,7 @@ class NodeReplaceVisitor(TraverserVisitor):
             # Unanalyzed types can have AST node references
             self.fixup_type(node.unanalyzed_type)
 
-    def process_type_var_def(self, tv: TypeVarDef) -> None:
+    def process_type_var_def(self, tv: TypeVarType) -> None:
         for value in tv.values:
             self.fixup_type(value)
         self.fixup_type(tv.upper_bound)
@@ -213,7 +213,7 @@ class NodeReplaceVisitor(TraverserVisitor):
             node.node = self.fixup(node.node)
             if isinstance(node.node, Var):
                 # The Var node may be an orphan and won't otherwise be processed.
-                fixup_var(node.node, self.replacements)
+                node.node.accept(self)
 
     def visit_namedtuple_expr(self, node: NamedTupleExpr) -> None:
         super().visit_namedtuple_expr(node)
@@ -266,6 +266,10 @@ class NodeReplaceVisitor(TraverserVisitor):
         self.fixup_type(node.type)
         super().visit_var(node)
 
+    def visit_type_alias(self, node: TypeAlias) -> None:
+        self.fixup_type(node.target)
+        super().visit_type_alias(node)
+
     # Helpers
 
     def fixup(self, node: SN) -> SN:
@@ -366,12 +370,13 @@ class TypeReplaceVisitor(SyntheticTypeVisitor[None]):
         if typ.fallback is not None:
             typ.fallback.accept(self)
         for tv in typ.variables:
-            tv.upper_bound.accept(self)
-            for value in tv.values:
-                value.accept(self)
+            if isinstance(tv, TypeVarType):
+                tv.upper_bound.accept(self)
+                for value in tv.values:
+                    value.accept(self)
 
     def visit_overloaded(self, t: Overloaded) -> None:
-        for item in t.items():
+        for item in t.items:
             item.accept(self)
         # Fallback can be None for overloaded types that haven't been semantically analyzed.
         if t.fallback is not None:
@@ -459,13 +464,6 @@ def replace_nodes_in_symbol_table(symbols: SymbolTable,
                 old = node.node
                 replace_object_state(new, old)
                 node.node = new
-            if isinstance(node.node, Var):
+            if isinstance(node.node, (Var, TypeAlias)):
                 # Handle them here just in case these aren't exposed through the AST.
-                # TODO: Is this necessary?
-                fixup_var(node.node, replacements)
-
-
-def fixup_var(node: Var, replacements: Dict[SymbolNode, SymbolNode]) -> None:
-    if node.type:
-        node.type.accept(TypeReplaceVisitor(replacements))
-    node.info = cast(TypeInfo, replacements.get(node.info, node.info))
+                node.node.accept(NodeReplaceVisitor(replacements))
